/**
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.xume.easybuilders.annotations;

import static org.xume.easybuilders.util.StringUtils.isNotEmpty;

import java.lang.reflect.Method;

import org.xume.easybuilders.api.BuildStep;
import org.xume.easybuilders.util.ReflectionUtils;


/**
 * @author Johan Siebens
 */
public final class CallBuildStepFactory extends AbstractMethodInvocationBuilderStepFactory<Call> {

	private Method method;

	public BuildStep getBuildStep() {
		return createMethodInvocationBuildStep(method);
	}

	public void initialize(Call call, Method builderMethod, Class<?> targetClass) {
		String methodName = defineMethodName(call, builderMethod);
		this.method = ReflectionUtils.findMethod(targetClass, methodName, resolveParameterTypes(builderMethod));
	}

	private String defineMethodName(Call call, Method method) {
		return isNotEmpty(call.value()) ? call.value() : method.getName();
	}

}
