/**
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.xume.easybuilders.annotations;

import static org.xume.easybuilders.util.StringUtils.isNotEmpty;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.xume.easybuilders.api.BuildStep;
import org.xume.easybuilders.api.ObjectBuilderConfigurationException;
import org.xume.easybuilders.util.ReflectionUtils;

/**
 * @author Johan Siebens
 */
public final class CallForEachBuildStepFactory extends AbstractMethodInvocationBuilderStepFactory<CallForEach> {

	private Method method;

	public BuildStep getBuildStep() {
		return new MultipleMethodInvocationBuildStep(method);
	}

	public void initialize(CallForEach callForEach, Method builderMethod, Class<?> targetClass) {
		Class<?>[] parameterTypes = builderMethod.getParameterTypes();
		if (parameterTypes.length != 1) {
			throw new ObjectBuilderConfigurationException("CallForEach methods should have exactly 1 parameter");
		}
		else if (!parameterTypes[0].isArray()) {
			throw new ObjectBuilderConfigurationException("CallForEach methods should have an array parameter");
		}

		Class<?>[] singleParamterType = { parameterTypes[0].getComponentType() };
		String methodName = defineMethodName(callForEach, builderMethod);
		this.method = ReflectionUtils.findMethod(targetClass, methodName, resolveParameterTypes(singleParamterType));
	}

	private String defineMethodName(CallForEach callForEach, Method method) {
		return isNotEmpty(callForEach.value()) ? callForEach.value() : method.getName();
	}

	private static class MultipleMethodInvocationBuildStep implements BuildStep {

		private List<BuildStep> buildSteps = new LinkedList<BuildStep>();

		private Method method;

		public MultipleMethodInvocationBuildStep(Method method) {
			this.method = method;
		}

		public void execute(Object target) {
			for (BuildStep step : buildSteps) {
				step.execute(target);
			}
		}

		public void record(Object... args) {
			buildSteps.clear();
			List<Object> list = Arrays.asList((Object[]) args[0]);
			for (Object arg : list) {
				BuildStep step = createMethodInvocationBuildStep(method);
				step.record(arg);
				this.buildSteps.add(step);
			}
		}
	}

}
